/* 
 * tclVmsTime.c --
 *
 *	Contains VMS specific versions of Tcl functions that
 *	obtain time values from the operating system.
 */

#include "tclInt.h"
#include "tclPort.h"

#include <starlet.h>
#include <unixlib.h>

#if 0
struct timeval {
    long	tv_sec;
    long	tv_usec;
};
#endif

#if 0
struct timezone {
    int tz_minuteswest;
    int tz_dsttime;
};
#endif

#define gettimeofday VMSgettimeofday

gettimeofday(struct timeval *tv, struct timezone *timezone)
{
    /*
     * Emulate unix's gettimeofday call
     *
     * This is not good enough since we round up
     * to the nearest second.
     */
    char tbuff[32];
    short lt;
    int flag = 1;			/* Return time, no date */
    long quadTime[2];
    struct {
    	int l;
    	void *a;
    } dt;

    sys$gettim(quadTime);
    tv->tv_sec = decc$fix_time(quadTime);

    /*
     * This is my attempt at extracting the fractional parts
     * of a second from a VMS quadtime. lib$ediv does not work
     * on a Vax if the quotient overflows.
     *
     * I assume that the fractional part of a second will not
     * change due to the convertions from VMS time to Unix time
     * since they have different epochs.
     */
    dt.l = sizeof(tbuff);
    dt.a = tbuff;
    sys$asctim(&lt, &dt, quadTime, flag);
    tbuff[lt-1] = 0;
    sscanf(tbuff, "%*d:%*d:%*d.%d", &tv->tv_usec);
    tv->tv_usec *= 10000;
}

/*
 *-----------------------------------------------------------------------------
 *
 * TclGetSeconds --
 *
 *	This procedure returns the number of seconds from the epoch.  On
 *	most Unix systems the epoch is Midnight Jan 1, 1970 GMT.
 *
 * Results:
 *	Number of seconds from the epoch.
 *
 * Side effects:
 *	None.
 *
 *-----------------------------------------------------------------------------
 */

unsigned long
TclGetSeconds()
{
    return time((time_t *) NULL);
}

/*
 *-----------------------------------------------------------------------------
 *
 * TclGetClicks --
 *
 *	This procedure returns a value that represents the highest resolution
 *	clock available on the system.  There are no garantees on what the
 *	resolution will be.  In Tcl we will call this value a "click".  The
 *	start time is also system dependant.
 *
 * Results:
 *	Number of clicks from some start time.
 *
 * Side effects:
 *	None.
 *
 *-----------------------------------------------------------------------------
 */

unsigned long
TclGetClicks()
{
    unsigned long now;
    struct timeval date;

    gettimeofday(&date, 0);
    now = date.tv_sec*1000000 + date.tv_usec;

    return now;
}

/*
 *----------------------------------------------------------------------
 *
 * TclGetTimeZone --
 *
 *	Determines the current timezone.  The method varies wildly
 *	between different platform implementations, so its hidden in
 *	this function.
 *
 * Results:
 *	Hours east of GMT.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

int
TclGetTimeZone (currentTime)
    unsigned long  currentTime;
{
    /*
     * Determine how a timezone is obtained from "struct tm".  If there is no
     * time zone in this struct (very lame) then use the timezone variable.
     * This is done in a way to make the timezone variable the method of last
     * resort, as some systems have it in addition to a field in "struct tm".
     * The gettimeofday system call can also be used to determine the time
     * zone.
     */

    /*
	Assume GMT
    */
    return 0;
}

/*
 *----------------------------------------------------------------------
 *
 * TclGetTime --
 *
 *	Gets the current system time in seconds and microseconds
 *	since the beginning of the epoch: 00:00 UCT, January 1, 1970.
 *
 * Results:
 *	Returns the current time in timePtr.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

void
TclGetTime(timePtr)
    Tcl_Time *timePtr;		/* Location to store time information. */
{
    struct timeval tv;
    
    (void) gettimeofday(&tv, 0);
    timePtr->sec = tv.tv_sec;
    timePtr->usec = tv.tv_usec;
}

